# frozen_string_literal: true

module Hyde
  # Abstract class that reacts to request navigation.
  # Does nothing by default, behaviour should be overriden through
  # #reject and #process
  # @abstract
  class Node
    # @param path [Object]
    def initialize(path)
      @pattern = Pattern.new(path).freeze
    end

    # Try to navigate the path. Run method callback in response.
    # @param [Hyde::Request]
    # @return [Boolean]
    def go(request)
      # rejected at pattern
      return reject(request) unless @pattern.match?(request.path)

      request.push_state
      request.path, splat, param = @pattern.match(request.path)
      request.splat.append(*splat)
      request.param.merge!(param)
      value = process(request)
      # rejected at callback - restore state
      request.pop_state unless value
      # finally, return process value
      value
    end

    # Method callback on failed request navigation
    # @param _request [Hyde::Request]
    # @return false
    def reject(_request)
      false
    end

    # Method callback on successful request navigation
    # @param _request [Hyde::Request]
    # @return true
    def process(_request)
      true
    end
  end
end
